#!pip install kubernetes

from kubernetes import client, config
import time

class FunctionController:
    def __init__(self):
        # Load Kubernetes configuration
        config.load_kube_config()

        # Initialize Kubernetes API client
        self.k8s_client = client.CoreV1Api()

    def create_function(self, function_name, image, replicas=1):
        # Define function deployment object
        deployment = client.V1Deployment()
        deployment.metadata = client.V1ObjectMeta(name=function_name)
        deployment.spec = client.V1DeploymentSpec(
            replicas=replicas,
            selector=client.V1LabelSelector(
                match_labels={"app": function_name}
            ),
            template=client.V1PodTemplateSpec(
                metadata=client.V1ObjectMeta(labels={"app": function_name}),
                spec=client.V1PodSpec(
                    containers=[client.V1Container(
                        name=function_name,
                        image=image,
                        ports=[client.V1ContainerPort(container_port=8080)]
                    )]
                )
            )
        )

        # Create function deployment
        self.k8s_client.create_namespaced_deployment(namespace='default', body=deployment)

    def scale_function(self, function_name, replicas):
        # Get function deployment
        deployment = self.k8s_client.read_namespaced_deployment(name=function_name, namespace='default')

        # Update replicas
        deployment.spec.replicas = replicas

        # Apply changes
        self.k8s_client.patch_namespaced_deployment(name=function_name, namespace='default', body=deployment)

if __name__ == "__main__":
    controller = FunctionController()

    # Example: Create a function
    controller.create_function(function_name="hello-python", image="your-registry/hello-python:latest")

    # Example: Scale a function
    controller.scale_function(function_name="hello-python", replicas=3)

    # Keep the controller running
    while True:
        time.sleep(60)  # Sleep to prevent high CPU usage

